#!/bin/env bash
#example wrapper script for automating miniBarcoder pipeline for 313-bp barcodes locally
#tested on Ubuntu 14.04 LTS and 16.04 LTS
#pre-requisites:
	# miniBarcoder (https://github.com/asrivathsan/miniBarcoder)
	# seqkit (https://github.com/shenwei356/seqkit), versions tested: v0.7.2 and v0.12.0
	# GNU Parallel, version tested "20161222"
	# NCBI nt database downloaded/updated
##============================================================================================

## TO RUN THE SCRIPT:
## sh SuppFileS2_miniBarcoder_wrapper.sh $1 $2 $3
	#$1 fastq file
	#$2 fasta file
	#$3 demultiplex sheet

##============================================================================================

##EDIT VARIABLES AS NEEDED:
export NUM_THREADS=4
export PATH_BASECALL=/insert/absolute/path/to/working/files/here
export PATH_BLASTDB=/insert/absolute/path/to/blast/db/here
export GENETIC_CODES="2\n4\n5\n9\n13"	#https://www.ncbi.nlm.nih.gov/Taxonomy/Utils/wprintgc.cgi
export JOBS=3	#jobs to run concurrently GNU Parallel: NUM_THREADS*JOBS = total CPU cores
export MAFFT=1_HS_MAFFT_10-3_hac_samereads_barcodes
export RACON=3_HS_RACON_10-3_hac_samereads_barcodes
export MAFFT_AA=2_HS_MAFFT_AA_10-3_hac_samereads_barcodes
export RACON_AA=4_HS_RACON_AA_10-3_hac_samereads_barcodes
export CONSOL=5_HS_consol_10-3_hac_samereads_barcodes
export FINAL=HS_MinION_10-3_hac_samereads_barcodes

##============================================================================================

cd ${HOME}/Applications/miniBarcoder
source ${HOME}/miniconda2/bin/activate mbconda

#convert the basecalled fastq file into fasta format for miniBarcoder.py
seqtk seq -A "$PATH_BASECALL"/$1 > "$PATH_BASECALL"/$2

#splice the sample list into the sub-groups by translation table for downstream amino acid correction
cut -d ',' -f 1 "$PATH_BASECALL"/$3 | grep -F 'Actinopterygii' > "$PATH_BASECALL"/Code2_samples.txt
cut -d ',' -f 1 "$PATH_BASECALL"/$3 | grep -e 'Cnidaria' -e 'Porifera' > "$PATH_BASECALL"/Code4_samples.txt
cut -d ',' -f 1 "$PATH_BASECALL"/$3 | grep -e 'Annelida' -e 'Arthropoda' -e 'Mollusca' -e 'Nemertea' -e 'Sipuncula' > "$PATH_BASECALL"/Code5_samples.txt
cut -d ',' -f 1 "$PATH_BASECALL"/$3 | grep -e 'Echinodermata' -e 'Hemichordata' -e 'Platyhelminthes' > "$PATH_BASECALL"/Code9_samples.txt
cut -d ',' -f 1 "$PATH_BASECALL"/$3 | grep -F 'Ascidiacea' > "$PATH_BASECALL"/Code13_samples.txt

##1_MAFFT_barcodes
python miniBarcoder.py -d "$PATH_BASECALL"/$3 -l 200 -m 2 -t "$NUM_THREADS" -o "$MAFFT" -f "$PATH_BASECALL"/$2
cp "$MAFFT"/all_barcodes.fa "$MAFFT"/"$MAFFT".fa
python filter_by_Ns.py -n 3 -i "$MAFFT"/"$MAFFT".fa

##3_RACON_barcodes
python racon_consensus.py -i "$MAFFT" -d "$RACON" -o "$RACON"/"$RACON".fa -b "$MAFFT"/"$MAFFT"_Nfilter.fa

##2_MAFFT_AA_barcodes
mkdir "$MAFFT_AA"_namino{1..3}
cp "$MAFFT"/"$MAFFT"_Nfilter.fa "$MAFFT_AA"_namino1/
cp "$MAFFT"/"$MAFFT"_Nfilter.fa "$MAFFT_AA"_namino2/
cp "$MAFFT"/"$MAFFT"_Nfilter.fa "$MAFFT_AA"_namino3/
echo -e "$GENETIC_CODES" | parallel -j "$JOBS" --env "$PATH_BLASTDB" 'seqkit grep -w 0 -r -f "$PATH_BASECALL"/Code{}_samples.txt "$MAFFT_AA"_namino1/"$MAFFT"_Nfilter.fa > "$MAFFT_AA"_namino1/"$MAFFT_AA"_Code{}.fa; python aacorrection.py -l 300 -L 330 -d "$PATH_BLASTDB" -b "$MAFFT_AA"_namino1/"$MAFFT_AA"_Code{}.fa -o "$MAFFT_AA"_namino1/"$MAFFT_AA"_namino1_Code{}.fa -p "$NUM_THREADS" -n 1 -g {}'
echo -e "$GENETIC_CODES" | parallel -j "$JOBS" --env "$PATH_BLASTDB" 'seqkit grep -w 0 -r -f "$PATH_BASECALL"/Code{}_samples.txt "$MAFFT_AA"_namino2/"$MAFFT"_Nfilter.fa > "$MAFFT_AA"_namino2/"$MAFFT_AA"_Code{}.fa; python aacorrection.py -l 300 -L 330 -d "$PATH_BLASTDB" -b "$MAFFT_AA"_namino2/"$MAFFT_AA"_Code{}.fa -o "$MAFFT_AA"_namino2/"$MAFFT_AA"_namino2_Code{}.fa -p "$NUM_THREADS" -n 2 -g {}'
echo -e "$GENETIC_CODES" | parallel -j "$JOBS" --env "$PATH_BLASTDB" 'seqkit grep -w 0 -r -f "$PATH_BASECALL"/Code{}_samples.txt "$MAFFT_AA"_namino3/"$MAFFT"_Nfilter.fa > "$MAFFT_AA"_namino3/"$MAFFT_AA"_Code{}.fa; python aacorrection.py -l 300 -L 330 -d "$PATH_BLASTDB" -b "$MAFFT_AA"_namino3/"$MAFFT_AA"_Code{}.fa -o "$MAFFT_AA"_namino3/"$MAFFT_AA"_namino3_Code{}.fa -p "$NUM_THREADS" -n 3 -g {}'
cat "$MAFFT_AA"_namino1/"$MAFFT_AA"_namino1_Code*.fa > "$MAFFT_AA"_namino1/"$MAFFT_AA"_namino1.fa
cat "$MAFFT_AA"_namino2/"$MAFFT_AA"_namino2_Code*.fa > "$MAFFT_AA"_namino2/"$MAFFT_AA"_namino2.fa
cat "$MAFFT_AA"_namino3/"$MAFFT_AA"_namino3_Code*.fa > "$MAFFT_AA"_namino3/"$MAFFT_AA"_namino3.fa

##4_RACON_AA_barcodes
mkdir "$RACON_AA"_namino{1..3}
cp "$RACON"/"$RACON".fa "$RACON_AA"_namino1/
cp "$RACON"/"$RACON".fa "$RACON_AA"_namino2/
cp "$RACON"/"$RACON".fa "$RACON_AA"_namino3/
echo -e "$GENETIC_CODES" | parallel -j "$JOBS" --env "$PATH_BLASTDB" 'seqkit grep -w 0 -r -f "$PATH_BASECALL"/Code{}_samples.txt "$RACON_AA"_namino1/"$RACON".fa > "$RACON_AA"_namino1/"$RACON_AA"_Code{}.fa; python aacorrection.py -l 300 -L 330 -d "$PATH_BLASTDB" -b "$RACON_AA"_namino1/"$RACON_AA"_Code{}.fa -o "$RACON_AA"_namino1/"$RACON_AA"_namino1_Code{}.fa -p "$NUM_THREADS" -n 1 -g {}'
echo -e "$GENETIC_CODES" | parallel -j "$JOBS" --env "$PATH_BLASTDB" 'seqkit grep -w 0 -r -f "$PATH_BASECALL"/Code{}_samples.txt "$RACON_AA"_namino2/"$RACON".fa > "$RACON_AA"_namino2/"$RACON_AA"_Code{}.fa; python aacorrection.py -l 300 -L 330 -d "$PATH_BLASTDB" -b "$RACON_AA"_namino2/"$RACON_AA"_Code{}.fa -o "$RACON_AA"_namino2/"$RACON_AA"_namino2_Code{}.fa -p "$NUM_THREADS" -n 2 -g {}'
echo -e "$GENETIC_CODES" | parallel -j "$JOBS" --env "$PATH_BLASTDB" 'seqkit grep -w 0 -r -f "$PATH_BASECALL"/Code{}_samples.txt "$RACON_AA"_namino3/"$RACON".fa > "$RACON_AA"_namino3/"$RACON_AA"_Code{}.fa; python aacorrection.py -l 300 -L 330 -d "$PATH_BLASTDB" -b "$RACON_AA"_namino3/"$RACON_AA"_Code{}.fa -o "$RACON_AA"_namino3/"$RACON_AA"_namino3_Code{}.fa -p "$NUM_THREADS" -n 3 -g {}'
cat "$RACON_AA"_namino1/"$RACON_AA"_namino1_Code*.fa > "$RACON_AA"_namino1/"$RACON_AA"_namino1.fa
cat "$RACON_AA"_namino2/"$RACON_AA"_namino2_Code*.fa > "$RACON_AA"_namino2/"$RACON_AA"_namino2.fa
cat "$RACON_AA"_namino3/"$RACON_AA"_namino3_Code*.fa > "$RACON_AA"_namino3/"$RACON_AA"_namino3.fa

##5_CONSOLIDATED_barcodes
mkdir "$CONSOL"_namino{1..3}
seq 1 3 | parallel --delay 1 -j "$JOBS" 'python scripts/consolidate.py -m "$MAFFT_AA"_namino{}/"$MAFFT_AA"_namino{}.fa -r "$RACON_AA"_namino{}/"$RACON_AA"_namino{}.fa -o "$CONSOL"_namino{}/"$CONSOL"_namino{}.fa -t "$CONSOL"_namino{}/tmp'

##============================================================================================

##centralise the MinION barcodes into one folder
mkdir "$FINAL"
cp "$MAFFT"/"$MAFFT".fa "$FINAL"/
cp "$MAFFT"/"$MAFFT"_Nfilter.fa "$FINAL"/
cp "$RACON"/"$RACON".fa "$FINAL"/
cp "$MAFFT_AA"_namino1/"$MAFFT_AA"_namino1.fa "$FINAL"/
cp "$MAFFT_AA"_namino2/"$MAFFT_AA"_namino2.fa "$FINAL"/
cp "$MAFFT_AA"_namino3/"$MAFFT_AA"_namino3.fa "$FINAL"/
cp "$RACON_AA"_namino1/"$RACON_AA"_namino1.fa "$FINAL"/
cp "$RACON_AA"_namino2/"$RACON_AA"_namino2.fa "$FINAL"/
cp "$RACON_AA"_namino3/"$RACON_AA"_namino3.fa "$FINAL"/
cp "$CONSOL"_namino1/"$CONSOL"_namino1.fa "$FINAL"/
cp "$CONSOL"_namino2/"$CONSOL"_namino2.fa "$FINAL"/
cp "$CONSOL"_namino3/"$CONSOL"_namino3.fa "$FINAL"/ 

#tally across MinION barcode datasets
grep -c '>' "$FINAL"/*.fa > "$FINAL"/"$FINAL"_counts.txt

##entire script took approximately 25 mins

